/* Dec 7, 1997
** C program for Salamander data - 
** Post-MCEM stage: estimation of error, Fisher info, and random effects
*/

#include ".../sal.h"
#include ".../trunc_normal.c"

int main()
{
  int nobs = SSIZE;
  void Fisher();

   read_data(".../salamander.dat", 1);
   cal_ybar();
   Fisher();
}

void Fisher()
{
   int M = 20000, m = 100;
   double vf = 50.0, vm = 50.0;
   double bf[N], bm[N];
   double *Bf[N], *Bm[N], *Bf_old[N], *Bm_old[N];
   double *Score[6], Info[6], Gamma[36];
   double alpha[4], alpha_old[4];
   double maxdiff ;
   long time1;
   int l,i,j,g,h,k;
   void Estep(long *t, double alpha[], double vf, double vm, int MC, int m,
          double *Bf[], double *Bm[], double bf[], double bm[]);
   void FisherInfo(double alpha[], double vf, double vm, int M, 
      double *Bf[], double *Bm[], double *Score[], double Info[], 
	  double Gamma[]);

   for (j = 0; j < N; ++j)
   {
      Bf[j] = malloc(Mmax*sizeof(double));
      Bm[j] = malloc(Mmax*sizeof(double));
   }

   for (g = 0; g < 6; ++g)
   {
	  Score[g] = malloc(Mmax*sizeof(double));
	  Info[g] = 0;
   }	
   for (g = 0; g < 6; ++g)
     for (h = 0; h < 6; ++h)
		 Gamma[6*g+h] = 0;
  
   time1 = -1056;

/* Initialize the parameters, random effects	*/

   vf = 1.3841; vm = 1.2376;
   alpha[0] =  1.0190; alpha[1] = 0.3214; 
   alpha[2] = -1.9398; alpha[3] = 0.9971;

   Estep(&time1, alpha, vf, vm, M, m, Bf, Bm, bf, bm);
   FisherInfo(alpha, vf, vm, M, Bf, Bm, Score, Info, Gamma);
/*   Gamma(Score, M);
   Randeff(Bf, Bm, M);
*/
}

void FisherInfo(double alpha[], double vf, double vm, int M, 
   double *Bf[], double *Bm[], double *Score[], double Info[], 
   double Gamma[])
{
   int g,h,i,k,l,j;
   double Sf, Sm, mu, sco[4], mscore[6], cp;


/* Calculate the score vector and the Info*/
  
   for(l = 0; l < M; ++l)
   {
       /* A: calculate Score[0-3][l], Info[0-3] */
       for (g= 0; g < 4; ++g)
	      sco[g] = 0;
       for (k = 0; k < SSIZE; ++k)     
       {
          mu =  1 -  1 / (1+exp(alpha[mix[k]]+ Bf[female[k]][l]
                        + Bm[male[k]][l]));
          sco[mix[k]] += -mu;
          Info[mix[k]] += mu*(1-mu);
       }
       for (g = 0; g < 4; ++g)
       {
           Score[g][l] = ybar[g]*nmix + sco[g];
       }

      /* B: calculate the bottom part of Score and Info */
      Sf = Sm = 0;
      for(j = 0; j < N; j++)
      {
          Sf += pow(Bf[j][l],2);
          Sm += pow(Bm[j][l],2);
      }
      Score[4][l] = Sf/(2*vf*vf) - N/(2*vf);
      Score[5][l] = Sm/(2*vm*vm) - N/(2*vm);
      Info[4] += Sf;
      Info[5] += Sm;
    }
    
    /*  Augmented data Information (diagonal matrix): */   
    for (g = 0; g < 4; ++g)
       Info[g] = Info[g] / M;
    Info[4] = Info[4]/(M*vf*vf*vf) - N/(2*vf*vf);
    Info[5] = Info[5]/(M*vm*vm*vm) - N/(2*vm*vm);
    printf("\nAugmented data info I_a:\n");
    for (g = 0; g < 6; ++g)
	   printf("%.6lf  ", Info[g]);
    printf("\n");

    /* Calculate Missing data info */
	printf("\nMissing data info:\n");
    for (g = 0; g < 6; g++)
	{
       for (h = 0; h < 6; ++h)
       {
           cp = 0; 
           for (l = 0; l < M; ++l)
			  cp += Score[g][l]*Score[h][l];
           printf("%.6lf  ", cp/M);
       }
	   printf("\n");
    }
           
            
/* Estimate Gamma - sum covariances up to lag 5*/

  /* calculate average score */
  for (g = 0; g < 6; ++g)
	  mscore[g] = 0;
  printf("\nMean score:\n");
  for (g = 0; g < 6; ++g)
  {
	 for (l = 0; l < M; ++l)
		mscore[g] += Score[g][l];
     mscore[g] = mscore[g] / M;
	 printf(" %.4lf", mscore[g]);
  }

  /* calculate covariances */
	printf("\nVariance of score:\n");
    for (g = 0; g < 6; g++)
	{
       for (h = 0; h < 6; ++h)
       {
           cp = 0; 
           for (l = 0; l < M; ++l)
			  cp += Score[g][l]*Score[h][l];
	       Gamma[6*g+h] = cp/M - mscore[g]*mscore[h];
           printf("%.3lf  ", Gamma[6*g+h]);
       }
	   printf("\n");
    }
   
	for (i = 1; i <= 5; ++i)
	{
	  printf("\n Lag %d autocovariance:\n", i);
	  for(g = 0; g < 6; g++)
	  {
		 for (h = 0; h < 6; ++h)
		 {
			cp = 0;
			for (l = 0; l < M-i; ++l)
			   cp += Score[g][l]*Score[h][l+i];
            Gamma[6*g+h] += cp/M - mscore[g]*mscore[h];
            Gamma[6*h+g] += cp/M - mscore[g]*mscore[h];
           printf("%.3lf  ", cp/M - mscore[g]*mscore[h]);
         }
		 printf("\n");
      }
    } 
            
   printf("\n Gamma:\n");
   for(g = 0; g < 6; g++)
   {
        for (h = 0; h < 6; ++h)
           printf("%.6lf  ", Gamma[6*g+h]); 
        printf("\n");
   }


}
	
   

 

void Estep(long *t, double alpha[], double vf, double vm, int MC, int m,
          double *Bf[], double *Bm[], double bf[], double bm[])
{ 
   /*  creates MC draws from (bf,bm) | alpha,vf,vm  */
   /*  to be stored in Bf,Bm                        */
   /*  m = burnout; *t = random seed                */
 
   int j,k,l;
   double bound, eta, u, v[SSIZE], sf, sm;
   double lowerf[N], lowerm[N], upperf[N], upperm[N];

/*   *t = 100;  */
   sf = sqrt(vf); sm = sqrt(vm);
   for (l = 0; l < MC+m; ++l)     /* MC for E step        */
   {
      for ( k= 0; k < SSIZE; ++k)    /* generate logistic var */
      {
         eta = exp(alpha[mix[k]] + bf[female[k]] + bm[male[k]]);
         if (y[k] == 1)
         {
            u = ran1(t) * eta/(1 + eta);
            v[k] = log(u/(1 - u));
         }
         else
         {
            u = ran1(t) / (1 + eta);
            v[k] = log((1-u)/u);
         }
      }

      for (j = 0; j < N; ++j)        /* init-ize bounds      */
      {
         lowerf[j] = -sf*LM;
         lowerm[j] = -sm*LM;
         upperf[j] = sf*LM;
         upperm[j] = sm*LM;
      }
      for (k = 0; k < SSIZE; ++k)    /* female bounds        */
      {
         bound = v[k] - alpha[mix[k]] - bm[male[k]];
         if (y[k] == 1)
            lowerf[female[k]]=(bound > lowerf[female[k]]) ?
                               bound : lowerf[female[k]];
         else
            upperf[female[k]]=(bound < upperf[female[k]]) ?
                              bound : upperf[female[k]];
       }

       for (j = 0; j < N; ++j)       /* female rand effects  */
         bf[j]=sf* r_tr_normal(t, lowerf[j]/sf, upperf[j]/sf);

       for (k = 0; k < SSIZE; ++k)   /* male bounds          */
       {
          bound = v[k] - alpha[mix[k]] - bf[female[k]];
          if (y[k] == 1)
             lowerm[male[k]]=(bound > lowerm[male[k]]) ?
                                bound : lowerm[male[k]];
          else
             upperm[male[k]]=(bound < upperm[male[k]]) ?
                               bound : upperm[male[k]];
       }

       for (j = 0; j < N; ++j)       /* male rand effects    */
         bm[j]=sm * r_tr_normal(t, lowerm[j]/sm, upperm[j]/sm);

       if (l >= m)
       {
          for (j = 0; j < N; ++j)
          {
             Bf[j][l-m] = bf[j];
             Bm[j][l-m] = bm[j];
          }
       }
    }

}

void read_data(char *filename, int table)
{
   int a;
   FILE *ifp;
   int k=0;

   ifp = fopen(filename,"r");
   while (fscanf(ifp, "%d", &a) == 1)
   {
      y[k] = a;
      fscanf(ifp,"%d",&a);
      female[k] = a-(table-1)*20-1;
      fscanf(ifp,"%d",&a);
      male[k] = a-(table-1)*20-1;
      if (female[k]%10 < 5 && male[k]%10 < 5)
         mix[k] = 0;
      else if (female[k]%10 < 5 && male[k]%10 > 4)
         mix[k] = 1;
      else if (female[k]%10 > 4 && male[k]%10 < 5)
         mix[k] = 2;
      else
         mix[k] = 3;
      ++k;
   }
    fclose(ifp);
}

void cal_ybar()
{
   int k,g;

   for (k = 0; k < SSIZE; ++k)
   {
      ybar[mix[k]] += y[k];
   }
   for (g = 0; g < 4; ++g)
   {
      ybar[g] = ybar[g]/nmix;
   }
}

