/* Dec 7, 1997
** C program for Salamander data - 
** Post-MCEM stage: estimation of error, Fisher info, and random effects
*/

#include ".../sal.h"
#include ".../trunc_normal.c"


int main()
{
  int nobs = SSIZE;
  void Fisher();

   read_data(".../salamander.dat", 1);
   cal_ybar();
   Fisher();
}

void Fisher()
{
   int M = 20000, m = 100;
   double sf = 1, sm = 1;
   double bf[N], bm[N];
   double *Bf[N], *Bm[N], *Bf_old[N], *Bm_old[N];
   double *Score[6], Info[15], Gamma[36];
   double alpha[4], alpha_old[4];
   double maxdiff ;
   long time1;
   int l,i,j,g,h,k;
   void Estep(long *t, double alpha[], double sf, double sm, int MC, int m,
          double *Bf[], double *Bm[], double bf[], double bm[]);
   void FisherInfo(double alpha[], double sf, double sm, int M, 
      double *Bf[], double *Bm[], double *Score[], double Info[], 
	  double Gamma[]);

   for (j = 0; j < N; ++j)
   {
      Bf[j] = malloc(Mmax*sizeof(double));
      Bm[j] = malloc(Mmax*sizeof(double));
   }

   for (g = 0; g < 6; ++g)
   {
	  Score[g] = malloc(Mmax*sizeof(double));
   }	
   for (g = 0; g < 15; ++g)
   {
	  Info[g] = 0;
   }	
   for (g = 0; g < 6; ++g)
     for (h = 0; h < 6; ++h)
		 Gamma[6*g+h] = 0;
  
   time1 = -1056;

/* Initialize the parameters, random effects	*/

   sf = 1.17697; sm = 1.11091;
   alpha[0] =  1.0180; alpha[1] = 0.3201; 
   alpha[2] = -1.9407; alpha[3] = 0.9942;

   Estep(&time1, alpha, sf, sm, M, m, Bf, Bm, bf, bm);
   FisherInfo(alpha, sf, sm, M, Bf, Bm, Score, Info, Gamma);
/*   Gamma(Score, M);
   Randeff(Bf, Bm, M);
*/
}

void FisherInfo(double alpha[], double sf, double sm, int M, 
   double *Bf[], double *Bm[], double *Score[], double Info[], 
   double Gamma[])
{
   int g,h,i,k,l,j;
   double Sf, Sm, mu, nu, eta, sco[6], mscore[6], cp;


/* Calculate the score vector and the Info*/
  
   for(l = 0; l < M; ++l)
   {
       /* A: calculate Score[0-3][l], Info */
       for (g= 0; g < 6; ++g)
	      sco[g] = 0;
       for (k = 0; k < SSIZE; ++k)     
       {
          mu =  1 -  1 / (1+exp(alpha[mix[k]]+ sf*Bf[female[k]][l]
                        + sm*Bm[male[k]][l]));
          eta = mu*(1-mu); 
		  nu = y[k] - mu;
          sco[mix[k]] += nu;
          sco[4] += nu*Bf[female[k]][l];
          sco[5] += nu*Bm[male[k]][l];
          Info[mix[k]] += eta;
          Info[mix[k]*2+4] += eta*Bf[female[k]][l];
          Info[mix[k]*2+5] += eta*Bm[male[k]][l];
          Info[12] += eta*Bf[female[k]][l]*Bf[female[k]][l];
          Info[13] += eta*Bf[female[k]][l]*Bm[male[k]][l];
          Info[14] += eta*Bm[male[k]][l]*Bm[male[k]][l];
       }
       for (g = 0; g < 6; ++g)
       {
           Score[g][l] = sco[g];
       }

    }
    
    /*  Augmented data Information (diagonal matrix): */   
    for (g = 0; g < 15; ++g)
       Info[g] = Info[g] / M;
    printf("\nAugmented data info I_a:\n");
    for (g = 0; g < 4; ++g)
	   printf("%.6lf  ", Info[g]);
    printf("\n");
    for (g = 0; g < 4; ++g)
	   printf("%.6lf  ", Info[2*g+4]);
    printf("\n");
    for (g = 0; g < 4; ++g)
	   printf("%.6lf  ", Info[2*g+5]);
    printf("\n");
    printf("%.6lf  %.6lf\n", Info[12], Info[13]);
    printf("%.6lf  %.6lf\n", Info[13], Info[14]);

    /* Calculate Missing data info */
	printf("\nMissing data info:\n");
    for (g = 0; g < 6; g++)
	{
       for (h = 0; h < 6; ++h)
       {
           cp = 0; 
           for (l = 0; l < M; ++l)
			  cp += Score[g][l]*Score[h][l];
           printf("%.6lf  ", cp/M);
       }
	   printf("\n");
    }
           
            
/* Estimate Gamma - sum covariances up to lag 5*/

  /* calculate average score */
  for (g = 0; g < 6; ++g)
	  mscore[g] = 0;
  printf("\nMean score:\n");
  for (g = 0; g < 6; ++g)
  {
	 for (l = 0; l < M; ++l)
		mscore[g] += Score[g][l];
     mscore[g] = mscore[g] / M;
	 printf(" %.4lf", mscore[g]);
  }

  /* calculate covariances */
	printf("\nVariance of score:\n");
    for (g = 0; g < 6; g++)
	{
       for (h = 0; h < 6; ++h)
       {
           cp = 0; 
           for (l = 0; l < M; ++l)
			  cp += Score[g][l]*Score[h][l];
	       Gamma[6*g+h] = cp/M - mscore[g]*mscore[h];
           printf("%.3lf  ", Gamma[6*g+h]);
       }
	   printf("\n");
    }
   
	for (i = 1; i <= 5; ++i)
	{
	  printf("\n Lag %d autocovariance:\n", i);
	  for(g = 0; g < 6; g++)
	  {
		 for (h = 0; h < 6; ++h)
		 {
			cp = 0;
			for (l = 0; l < M-i; ++l)
			   cp += Score[g][l]*Score[h][l+i];
            Gamma[6*g+h] += cp/M - mscore[g]*mscore[h];
            Gamma[6*h+g] += cp/M - mscore[g]*mscore[h];
           printf("%.3lf  ", cp/M - mscore[g]*mscore[h]);
         }
		 printf("\n");
      }
    } 
            
   printf("\n Gamma:\n");
   for(g = 0; g < 6; g++)
   {
        for (h = 0; h < 6; ++h)
           printf("%.6lf  ", Gamma[6*g+h]); 
        printf("\n");
   }


}
	
   

 

void Estep(long *t, double alpha[], double sf, double sm, int MC, int m,
          double *Bf[], double *Bm[], double bf[], double bm[])
{
   /*  creates MC draws from (bf,bm) | alpha,sf,sm  */
   /*  to be stored in Bf,Bm                        */
   /*  m = burnout; *t = random seed                */

   int j,k,l;
   double bound, eta, u, v[SSIZE];
   double lowerf[N], lowerm[N], upperf[N], upperm[N];

/*   *t = 100;  */
   for (l = 0; l < MC+m; ++l)     /* MC for E step        */
   {
      for ( k= 0; k < SSIZE; ++k)    /* generate logistic var */
      {
         eta = exp(alpha[mix[k]] + sf*bf[female[k]] + sm*bm[male[k]]);
         if (y[k] == 1)
         {
            u = ran1(t) * eta/(1 + eta);
            v[k] = log(u/(1 - u));
         }
         else
         {
            u = ran1(t) / (1 + eta);
            v[k] = log((1-u)/u);
         }
      }

      for (j = 0; j < N; ++j)        /* init-ize bounds      */
      {
         lowerf[j] = -LM;
         lowerm[j] = -LM;
         upperf[j] = LM;
         upperm[j] = LM;
      }
      for (k = 0; k < SSIZE; ++k)    /* female bounds        */
      {
         bound = (v[k] - alpha[mix[k]] - sm*bm[male[k]])/sf;
         if (y[k] == 1)
            lowerf[female[k]]=(bound > lowerf[female[k]]) ?
                               bound : lowerf[female[k]];
         else
            upperf[female[k]]=(bound < upperf[female[k]]) ?
                              bound : upperf[female[k]];
       }

       for (j = 0; j < N; ++j)       /* female rand effects  */
       {
         bf[j]= r_tr_normal(t, lowerf[j], upperf[j]);
       }

       for (k = 0; k < SSIZE; ++k)   /* male bounds          */
       {
          bound = (v[k] - alpha[mix[k]] - sf*bf[female[k]])/sm;
          if (y[k] == 1)
             lowerm[male[k]]=(bound > lowerm[male[k]]) ?
                                bound : lowerm[male[k]];
          else
             upperm[male[k]]=(bound < upperm[male[k]]) ?
                               bound : upperm[male[k]];
       }

       for (j = 0; j < N; ++j)       /* male rand effects    */
       {
         bm[j]= r_tr_normal(t, lowerm[j], upperm[j]);
       }


       if (l >= m)
       {
          for (j = 0; j < N; ++j)
          {
             Bf[j][l-m] = bf[j];
             Bm[j][l-m] = bm[j];
          }
       }
    }
}

void read_data(char *filename, int table)
{
   int a;
   FILE *ifp;
   int k=0;

   ifp = fopen(filename,"r");
   while (fscanf(ifp, "%d", &a) == 1)
   {
      y[k] = a;
      fscanf(ifp,"%d",&a);
      female[k] = a-(table-1)*20-1;
      fscanf(ifp,"%d",&a);
      male[k] = a-(table-1)*20-1;
      if (female[k]%10 < 5 && male[k]%10 < 5)
         mix[k] = 0;
      else if (female[k]%10 < 5 && male[k]%10 > 4)
         mix[k] = 1;
      else if (female[k]%10 > 4 && male[k]%10 < 5)
         mix[k] = 2;
      else
         mix[k] = 3;
      ++k;
   }
    fclose(ifp);
}

void cal_ybar()
{
   int k,g;

   for (k = 0; k < SSIZE; ++k)
   {
      ybar[mix[k]] += y[k];
   }
   for (g = 0; g < 4; ++g)
   {
      ybar[g] = ybar[g]/nmix;
   }
}

