#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

#include ".../erfcc.c"
#include ".../ran1.c"
#include ".../gasdev.c"

#define C0 2.515517
#define C1 0.802853
#define C2 0.010328
#define D1 1.432788
#define D2 0.189269
#define D3 0.001308
#define SQRT2 1.4142135623731
#define MINRANGE 1e-45

float r_tr_normal(long *idum, float a, float b);
float invinorm(float u);

/*
int main()
{
  float y;
  long time1;
  time1 = -time(NULL);
  y = r_tr_normal(&time1,-117,-14);
  printf(" %.4lf  ", y);
}
*/

float r_tr_normal(long *idum, float a, float b)
{
   float phi_a, phi_b;
   float y, tmp;
   int i, j, sign = 1 ;
   float u_tilt;

   if (a > 0 & b > 0)
   {
     sign = -1;  tmp = a; a = -b; b = -tmp;
   }
   phi_a = erfcc(-a/SQRT2)/2;
   phi_b = erfcc(-b/SQRT2)/2;
   if ((phi_b - phi_a) >= 0.3)
      do { 
         y = gasdev(idum);
      } while (y<a || y>b);
   else
   {
      u_tilt = (phi_b - phi_a)*ran1(idum)+phi_a;
      if (u_tilt < MINRANGE)
        y = (sign == 1) ? b: (-a);
      else
        y = invinorm(u_tilt);
   }
   return sign*y;
}

float invinorm(float u)
{
   float t, x, v;

   if (u == 0.5)
      return 0;
   else if (u > 0.5)
      v = 1 - u;
   else 
      v = u;
   t = sqrt(log(1/pow(v,2)));
   x = t - (C0+C1*t+C2*pow(t,2))/(1+D1*t+D2*pow(t,2)+D3*pow(t,3));
   if (u < 0.5)
      x = -x;
   return x;
}

/*
 The program is based on the rational approximation for the tails of the
 normal CDF from Abramowitz & Stegun, "Handbook of Mathematical Functions",
 Dover 1973, formula 26.2.23 p.933
*/
